@@ -26,6 +26,7 @@ public:
 	virtual void shutdown(void);
 
 	virtual size_t getNumElements(void);
+	virtual size_t getNumWaiters();
 
 	virtual void addThrottler(IPipeThrottler *throttler);
 	virtual void addOutgoingThrottler(IPipeThrottler *throttler);
@@ -36,6 +37,9 @@ public:
 
 	virtual bool Flush( int timeoutms=-1 );
 
+	virtual void setUsageString(const std::string& str);
+
+	virtual bool setCompressionSettings(const SCompressionSettings& params);
 
 private:
 	std::deque<std::string> queue;
@@ -43,6 +47,7 @@ private:
 	IMutex *mutex;
 	ICondition *cond;
 
+	size_t waiters;
 	bool has_error;
 };
 
