@@ -35,6 +35,7 @@
 #include "Interface/PluginMgr.h"
 #include "Interface/Thread.h"
 #include "Interface/DatabaseFactory.h"
+#include "Interface/WebSocket.h"
 
 #include "Server.h"
 #include "Template.h"
@@ -94,6 +95,16 @@
 #include <mach/mach.h>
 #endif
 
+#ifdef HAVE_SYS_RANDOM_H
+#if defined(__FreeBSD__)
+extern "C" {
+#include <sys/random.h>
+}
+#else
+#include <sys/random.h>
+#endif //__FreeBSD__
+#endif
+
 
 const size_t SEND_BLOCKSIZE=8192;
 const size_t MAX_THREAD_ID=std::string::npos;
@@ -165,6 +176,7 @@ CServer::CServer()
 	
 	log_mutex=createMutex();
 	action_mutex=createMutex();
+	web_socket_mutex = createMutex();
 	requests_mutex=createMutex();
 	outputs_mutex=createMutex();
 	db_mutex=createMutex();
@@ -210,6 +222,7 @@ void CServer::setup(void)
 #endif
 
 	CQuery::init_mutex();
+	CStreamPipe::init_mutex();
 
 #ifdef MODE_WIN
 	File::init_mutex();
@@ -343,6 +356,7 @@ CServer::~CServer()
 	
 	destroy(log_mutex);
 	destroy(action_mutex);
+	destroy(web_socket_mutex);
 	destroy(requests_mutex);
 	destroy(outputs_mutex);
 	destroy(db_mutex);
@@ -415,24 +429,24 @@ void CServer::Log( const std::string &pStr, int LogLevel)
 
 		if(log_console_time)
 		{
-			std::cout << buffer;
+				std::cout << buffer;
 		}
 
 		if( LogLevel==LL_ERROR )
 		{
-			std::cout << "ERROR: " << pStr << std::endl;
+				std::cout << "ERROR: " << pStr << std::endl;
 			if(logfile_a)
 				logfile << buffer << "ERROR: " << pStr << std::endl;
 		}
 		else if( LogLevel==LL_WARNING )
 		{
-			std::cout << "WARNING: " << pStr << std::endl;
+				std::cout << "WARNING: " << pStr << std::endl;
 			if(logfile_a)
 				logfile<< buffer << "WARNING: " << pStr << std::endl;
 		}
 		else
 		{
-			std::cout << pStr << std::endl;		
+				std::cout << pStr << std::endl;		
 			if(logfile_a)
 				logfile << buffer << pStr << std::endl;
 		}
@@ -1046,7 +1060,7 @@ IPipe* CServer::ConnectStream(std::string pServer, unsigned short pPort, unsigne
 
 	for (size_t i = 0; i < lookup_result.size(); ++i)
 	{
-		IPipe* ret = ConnectStream(lookup_result[i], pPort, pTimeoutms);
+		IPipe* ret = ConnectStream(pServer, lookup_result[i], pPort, pTimeoutms);
 		if (ret != NULL)
 			return ret;
 	}
@@ -1055,7 +1069,7 @@ IPipe* CServer::ConnectStream(std::string pServer, unsigned short pPort, unsigne
 }
 
 
-IPipe* CServer::ConnectStream(const SLookupBlockingResult& lookup_result, unsigned short pPort, unsigned int pTimeoutms)
+IPipe* CServer::ConnectStream(const std::string& connect_str, const SLookupBlockingResult& lookup_result, unsigned short pPort, unsigned int pTimeoutms)
 {
 	union
 	{
@@ -1148,12 +1162,12 @@ IPipe* CServer::ConnectStream(const SLookupBlockingResult& lookup_result, unsign
 	}
 	else
 	{
-		return new CStreamPipe(s);
+		return new CStreamPipe(s, "tcp " + connect_str);
 	}
 #else
 	if(rc!=SOCKET_ERROR)
 	{
-		return new CStreamPipe(s);
+		return new CStreamPipe(s, "tcp "+connect_str);
 	}
 #endif
 
@@ -1202,7 +1216,7 @@ IPipe* CServer::ConnectStream(const SLookupBlockingResult& lookup_result, unsign
 			if(recv_window_size>0)
 				setsockopt(s, SOL_SOCKET, SO_RCVBUF, (char *) &recv_window_size, sizeof(recv_window_size));
 #endif
-		    return new CStreamPipe(s);
+		    return new CStreamPipe(s, "tcp "+connect_str);
 		}
 	}
 	else
@@ -1237,6 +1251,8 @@ IPipe * CServer::ConnectSslStream(const std::string & pServer, unsigned short pP
 		return NULL;
 	}
 
+	bpipe->setUsageString("ssl " + pServer);
+
 	return ssl_pipe;
 #else
 	return NULL;
@@ -1245,7 +1261,7 @@ IPipe * CServer::ConnectSslStream(const std::string & pServer, unsigned short pP
 
 IPipe *CServer::PipeFromSocket(SOCKET pSocket)
 {
-	return new CStreamPipe(pSocket);
+	return new CStreamPipe(pSocket, "from socket");
 }
 
 void CServer::DisconnectStream(IPipe *pipe)
@@ -1266,6 +1282,11 @@ std::string CServer::LookupHostname(const std::string & pIp)
 	return hostname;
 }
 
+std::vector<std::string> CServer::getStreamPipeList()
+{
+	return CStreamPipe::getPipeList();
+}
+
 bool CServer::RegisterPluginPerThreadModel(IPluginMgr *pPluginMgr, std::string pName)
 {
 	IScopedLock lock(plugin_mutex);
@@ -1539,7 +1560,7 @@ bool CServer::createThread(IThread *thread, const std::string& name, CreateThrea
 
 #ifndef _LP64
 	//Only on 32bit architectures
-	pthread_attr_setstacksize(&attr, 1*1024*1024);
+	pthread_attr_setstacksize(&attr, 1 * 1024 * 1024);
 #endif
 
 #else
@@ -1668,7 +1689,7 @@ void CServer::wait(unsigned int ms)
 #ifdef _WIN32
 	Sleep(ms);
 #else
-	usleep(ms*1000);
+		usleep(ms * 1000);
 #endif
 }
 
@@ -2052,61 +2073,39 @@ unsigned int CServer::getSecureRandomNumber(void)
 {
 #ifdef _WIN32
 	unsigned int rnd;
-	errno_t err=rand_s(&rnd);
-	if(err!=0)
+	errno_t err = rand_s(&rnd);
+	if (err != 0)
 	{
 		Log("Error generating secure random number", LL_ERROR);
+		abort();
 		return getRandomNumber();
 	}
 	return rnd;
 #else
 	unsigned int rnd;
-	std::fstream rnd_in("/dev/urandom", std::ios::in | std::ios::binary );
-	if(!rnd_in.is_open())
-	{
-		Log("Error opening /dev/urandom for secure random number", LL_ERROR);
-		return getRandomNumber();
-	}
-	rnd_in.read((char*)&rnd, sizeof(unsigned int));
-	assert(rnd_in.gcount()==sizeof(unsigned int));
-	if(rnd_in.fail() || rnd_in.eof() )
-	{
-		Log("Error reading secure random number", LL_ERROR);
-		return getRandomNumber();
-	}
+	secureRandomFill(reinterpret_cast<char*>(&rnd), sizeof(rnd));
 	return rnd;
 #endif
 }
 
 std::vector<unsigned int> CServer::getSecureRandomNumbers(size_t n)
 {
-#ifndef _WIN32
-	std::fstream rnd_in("/dev/urandom", std::ios::in | std::ios::binary );
-	if(!rnd_in.is_open())
+	if (n == 0)
 	{
-		Log("Error opening /dev/urandom for secure random number", LL_ERROR);
-		return getRandomNumbers(n);
-	}
-#endif
-	std::vector<unsigned int> ret;
-	ret.resize(n);
-	for(size_t i=0;i<n;++i)
-	{
-#ifdef _WIN32
-		ret[i]=getSecureRandomNumber();
-#else
-		unsigned int rnd;
-		rnd_in.read((char*)&rnd, sizeof(unsigned int));
-		ret[i]=rnd;
-#endif		
+		return std::vector<unsigned int>();
 	}
 
-#ifndef _WIN32
-	if(rnd_in.fail() || rnd_in.eof() )
+	std::vector<unsigned int> ret;
+	ret.resize(n);
+#ifdef _WIN32
+	for (size_t i = 0; i<n; ++i)
 	{
-		Log("Error reading secure random numbers", LL_ERROR);
-		return getRandomNumbers(n);
-	}
+		ret[i] = getSecureRandomNumber();
+	}	
+#else
+	char* buf = reinterpret_cast<char*>(&ret[0]);
+	size_t bsize = sizeof(unsigned int)*n;
+	secureRandomFill(buf, bsize);
 #endif
 	return ret;
 }
@@ -2117,34 +2116,46 @@ void CServer::secureRandomFill(char *buf, size_t blen)
 	char *dptr=buf+blen;
 	while(buf<dptr)
 	{
-		if(dptr-buf>=sizeof(unsigned long))
+		const unsigned int rnd = getSecureRandomNumber();
+		const size_t to_write = (std::min)(sizeof(rnd), static_cast<size_t>(dptr - buf));
+		memcpy(buf, &rnd, to_write);
+		buf += to_write;
+	}
+#elif defined(HAVE_GETRANDOM) && defined(HAVE_SYS_RANDOM_H)
+	while (blen > 0)
+	{
+		ssize_t rc = getrandom(buf, blen, 0);
+		if (rc == -1
+			&& errno == EINTR)
 		{
-			*((unsigned long*)buf)=getSecureRandomNumber();
-			buf+=sizeof(unsigned long);
+			rc = 0;
 		}
-		else
+		else if (rc == -1)
 		{
-			unsigned long rnd=getSecureRandomNumber();
-			memcpy(buf, &rnd, dptr-buf);
-			buf+=dptr-buf;
+			Server->Log("Error filling secure random buffer. Errno " + convert((int64)errno), LL_ERROR);
+			abort();
 		}
+		blen -= rc;
+		buf += rc;
 	}
 #else
-	std::fstream rnd_in("/dev/urandom", std::ios::in | std::ios::binary );
-	if(!rnd_in.is_open())
+	std::fstream rnd_in("/dev/urandom", std::ios::in | std::ios::binary);
+	if (!rnd_in.is_open())
 	{
-		Log("Error opening /dev/urandom for secure random number fill", LL_ERROR);
+		Log("Error opening /dev/urandom for secure random number fill. Errno "+convert((int64)errno), LL_ERROR);
 		randomFill(buf, blen);
 		return;
 	}
 
 	rnd_in.read(buf, blen);
 
-	assert(rnd_in.gcount()==blen);
+	assert(rnd_in.gcount() == blen);
 
-	if(rnd_in.fail() || rnd_in.eof() )
+	if(rnd_in.gcount() != blen 
+		|| rnd_in.fail() || rnd_in.eof() )
 	{
-		Log("Error reading secure random numbers fill", LL_ERROR);
+		Log("Error reading secure random numbers fill. Errno " + convert((int64)errno), LL_ERROR);
+		abort();
 		randomFill(buf, blen);
 		return;
 	}
@@ -2267,3 +2278,29 @@ int CServer::getRecvWindowSize()
 	return recv_window_size;
 }
 #endif
+
+void CServer::addWebSocket(IWebSocket* websocket)
+{
+	IScopedLock lock(web_socket_mutex);
+
+	web_sockets[websocket->getName()] = websocket;
+}
+
+THREAD_ID CServer::ExecuteWebSocket(const std::string& name, str_map& GET, str_map& PARAMS, IPipe* pipe, const std::string& endpoint_name)
+{
+	IWebSocket* ws;
+	{
+		IScopedLock lock(web_socket_mutex);
+		std::map<std::string, IWebSocket*>::iterator it = web_sockets.find(name);
+		if (it == web_sockets.end())
+			return ILLEGAL_THREAD_ID;
+
+		ws = it->second;
+	}
+
+	THREAD_ID tid = getThreadID();
+
+	ws->Execute(GET, tid, PARAMS, pipe, endpoint_name);
+
+	return tid;
+}
