@@ -90,7 +90,7 @@
 
 CClientThread::CClientThread(SOCKET pSocket, CTCPFileServ* pParent)
 	: extra_buffer(NULL), waiting_for_chunk(false),
-	backup_semantics(true)
+	backup_semantics(true), is_tunneled(false)
 {
 	int_socket=pSocket;
 
@@ -138,7 +138,7 @@ CClientThread::CClientThread(SOCKET pSocket, CTCPFileServ* pParent)
 }
 
 CClientThread::CClientThread(IPipe *pClientpipe, CTCPFileServ* pParent, std::vector<char>* extra_buffer)
-	: extra_buffer(extra_buffer), waiting_for_chunk(false)
+	: extra_buffer(extra_buffer), waiting_for_chunk(false), is_tunneled(true)
 {
 	stopped=false;
 	killable=false;
@@ -159,6 +159,9 @@ CClientThread::CClientThread(IPipe *pClientpipe, CTCPFileServ* pParent, std::vec
 	chunk_send_thread_ticket=ILLEGAL_THREADPOOL_TICKET;
 
 	stack.setAddChecksum(true);
+	IPipe::SCompressionSettings comp_params;
+	comp_params.mem = IPipe::Compression_HighMem;
+	pClientpipe->setCompressionSettings(comp_params);
 }
 
 CClientThread::~CClientThread()
@@ -280,6 +283,11 @@ bool CClientThread::RecvMessage()
 		if(extra_buffer==NULL || extra_buffer->empty())
 		{
 			rc=(_i32)clientpipe->Read(buffer, BUFFERSIZE, 60*1000);
+
+			if (rc > 0)
+			{
+				Server->Log("FileSRV received " + convert(rc));
+			}
 		}
 		else
 		{
@@ -356,9 +364,9 @@ bool CClientThread::ProcessPacket(CRData *data)
 #ifdef CHECK_IDENT
 				std::string ident;
 				data->getStr(&ident);
-				if(!FileServ::checkIdentity(ident))
+				if(!FileServ::checkIdentity(ident, is_tunneled))
 				{
-					Log("Identity check failed -2", LL_DEBUG);
+					Log("Identity check failed -2 ("+ ident+")", LL_DEBUG);
 					return false;
 				}
 #endif
@@ -1702,7 +1710,7 @@ bool CClientThread::GetFileBlockdiff(CRData *data, bool with_metadata)
 #ifdef CHECK_IDENT
 	std::string ident;
 	data->getStr(&ident);
-	if(!FileServ::checkIdentity(ident))
+	if(!FileServ::checkIdentity(ident, is_tunneled))
 	{
 		Log("Identity check failed -2", LL_DEBUG);
 		return false;
@@ -2098,7 +2106,7 @@ bool CClientThread::GetFileHashAndMetadata( CRData* data )
 #ifdef CHECK_IDENT
 	std::string ident;
 	data->getStr(&ident);
-	if(!FileServ::checkIdentity(ident))
+	if(!FileServ::checkIdentity(ident, is_tunneled))
 	{
 		Log("Identity check failed -hash", LL_DEBUG);
 		return false;
@@ -2400,7 +2408,7 @@ bool CClientThread::InformMetadataStreamEnd( CRData * data )
 #ifdef CHECK_IDENT
 	std::string ident;
 	data->getStr(&ident);
-	if(!FileServ::checkIdentity(ident))
+	if(!FileServ::checkIdentity(ident, is_tunneled))
 	{
 		Log("Identity check failed -hash", LL_DEBUG);
 		return false;
@@ -2433,7 +2441,7 @@ bool CClientThread::StopPhash(CRData * data)
 #ifdef CHECK_IDENT
 	std::string ident;
 	data->getStr(&ident);
-	if (!FileServ::checkIdentity(ident))
+	if (!FileServ::checkIdentity(ident, is_tunneled))
 	{
 		Log("Identity check failed -hash", LL_DEBUG);
 		return false;
@@ -2469,7 +2477,7 @@ bool CClientThread::FinishScript( CRData * data )
 #ifdef CHECK_IDENT
 	std::string ident;
 	data->getStr(&ident);
-	if(!FileServ::checkIdentity(ident))
+	if(!FileServ::checkIdentity(ident, is_tunneled))
 	{
 		Log("Identity check failed -FinishScript", LL_DEBUG);
 		return false;
