@@ -32,6 +32,8 @@ class Backup;
 class ServerBackupDao;
 class ImageBackup;
 class ServerChannelThread;
+class IECDHKeyExchange;
+class IECIESDecryption;
 
 const int c_group_default = 0;
 const int c_group_continuous = 1;
@@ -49,7 +51,8 @@ struct SProtocolVersions
 				symbit_version(0), phash_version(0),
 				wtokens_version(0), update_vols(0),
 				update_capa_interval(0), require_previous_cbitmap(0),
-				async_index_version(0)
+				async_index_version(0), restore_version(0),
+				filesrvtunnel(0)
 			{
 
 			}
@@ -74,6 +77,8 @@ struct SProtocolVersions
 	int update_vols;
 	int update_capa_interval;
 	std::string os_simple;
+	int restore_version;
+	int filesrvtunnel;
 };
 
 struct SRunningBackup
@@ -120,6 +125,11 @@ struct SShareCleanup
 	bool remove_callback;
 };
 
+namespace
+{
+	const int CAPA_NO_IMAGE_BACKUPS = 1;
+}
+
 class ClientMain : public IThread, public FileClientChunked::ReconnectionCallback,
 	public FileClient::ReconnectionCallback, public INotEnoughSpaceCallback,
 	public FileClient::NoFreeSpaceCallback, public FileClientChunked::NoFreeSpaceCallback,
@@ -145,10 +155,10 @@ public:
 		bool internet_connection;
 	};
 
-	bool sendClientMessage(const std::string &msg, const std::string &retok, const std::string &errmsg, unsigned int timeout, bool logerr=true, int max_loglevel=LL_ERROR, bool *retok_err=NULL, std::string* retok_str=NULL, SConnection* conn=NULL);
-	bool sendClientMessageRetry(const std::string &msg, const std::string &retok, const std::string &errmsg, unsigned int timeout, size_t retry=0, bool logerr=true, int max_loglevel=LL_ERROR, bool *retok_err=NULL, std::string* retok_str=NULL);
-	std::string sendClientMessage(const std::string &msg, const std::string &errmsg, unsigned int timeout, bool logerr=true, int max_loglevel=LL_ERROR, SConnection* conn=NULL);
-	std::string sendClientMessageRetry(const std::string &msg, const std::string &errmsg, unsigned int timeout, size_t retry=0, bool logerr=true, int max_loglevel=LL_ERROR, unsigned int timeout_after_first=0);
+	bool sendClientMessage(const std::string &msg, const std::string &retok, const std::string &errmsg, unsigned int timeout, bool logerr=true, int max_loglevel=LL_ERROR, bool *retok_err=NULL, std::string* retok_str=NULL, SConnection* conn=NULL, bool do_encrypt = true);
+	bool sendClientMessageRetry(const std::string &msg, const std::string &retok, const std::string &errmsg, unsigned int timeout, size_t retry=0, bool logerr=true, int max_loglevel=LL_ERROR, bool *retok_err=NULL, std::string* retok_str=NULL, bool do_encrypt = true);
+	std::string sendClientMessage(const std::string &msg, const std::string &errmsg, unsigned int timeout, bool logerr=true, int max_loglevel=LL_ERROR, SConnection* conn=NULL, bool do_encrypt = true);
+	std::string sendClientMessageRetry(const std::string &msg, const std::string &errmsg, unsigned int timeout, size_t retry=0, bool logerr=true, int max_loglevel=LL_ERROR, unsigned int timeout_after_first=0, bool do_encrypt=true);
 	void sendToPipe(const std::string &msg);
 
 	bool createDirectoryForClient();
@@ -166,7 +176,7 @@ public:
 	static bool tooManyClients(IDatabase *db, const std::string &clientname, ServerSettings *server_settings);
 	static int getClientID(IDatabase *db, const std::string &clientname, ServerSettings *server_settings, bool *new_client, std::string* authkey=NULL, int* client_group=NULL);
 
-	IPipe *getClientCommandConnection(ServerSettings* server_settings, int timeoutms=10000, std::string* clientaddr=NULL);
+	IPipe *getClientCommandConnection(ServerSettings* server_settings, int timeoutms=10000, std::string* clientaddr=NULL, bool do_encrypt=true, bool allow_reauth=false, bool* require_reauth = NULL);
 
 	virtual IPipe * new_fileclient_connection(void);
 
@@ -204,6 +214,10 @@ public:
 	}
 
 	std::string getIdentity();
+
+	std::string getSecretSessionKey();
+
+	std::string getSessionCompression(int& level);
 	
 	int getCurrImageVersion()
 	{
@@ -257,6 +271,8 @@ public:
 		return allow_restore_clients;
 	}
 
+	static void wakeupClientUidReset();
+
 private:
 	void unloadSQL(void);
 	void prepareSQL(void);
@@ -270,7 +286,7 @@ private:
 	void sendClientBackupIncrIntervall(void);
 	void sendSettings(void);
 	bool getClientSettings(bool& doesnt_exist);
-	bool updateClientSetting(const std::string &key, const std::string &value);
+	bool updateClientSetting(const std::string &key, const std::string &value, int use, int64 use_last_modified, bool allow_use_mod);
 	void sendClientLogdata(void);
 	bool isRunningImageBackup(const std::string& letter);
 	bool isImageGroupQueued(const std::string& letter, bool full);
@@ -295,6 +311,7 @@ private:
 	bool pauseRetryBackup();
 
 	bool sendServerIdentity(bool retry_exit);
+	bool authenticatePubKeyInt(IECDHKeyExchange* ecdh_key_exchange);
 	bool authenticatePubKey();
 
 	void timeoutRestores();
@@ -308,6 +325,8 @@ private:
 	bool renameClient(const std::string& clientuid);
 	void updateVirtualClients();
 
+	bool checkClientName(bool& continue_start_backups);
+
 
 	struct SPathComponents
 	{
@@ -372,6 +391,7 @@ private:
 	bool use_tmpfiles_images;
 
 	CTCPStack tcpstack;
+	CTCPStack tcpstack_checksum;
 
 	IMutex* throttle_mutex;
 	IPipeThrottler *client_throttler;
@@ -389,6 +409,9 @@ private:
 
 	std::string session_identity;
 	int64 session_identity_refreshtime;
+	std::string secret_session_key;
+	std::string session_compressed;
+	int session_compression_level;
 
 	ServerBackupDao* backup_dao;
 
@@ -424,4 +447,10 @@ private:
 	volatile bool do_reauthenticate;
 
 	std::vector<std::string> allow_restore_clients;
+
+	static IMutex* ecdh_key_exchange_mutex;
+	static std::vector<std::pair<IECDHKeyExchange*, int64> > ecdh_key_exchange_buffer;
+
+	static IMutex* client_uid_reset_mutex;
+	static ICondition* client_uid_reset_cond;
 };
