@@ -413,13 +413,19 @@ void InternetServiceConnector::ReceivePackets(IRunOtherCallback* run_other)
 
 								if(id==ID_ISC_AUTH2)
 								{
-									salt+=ecdh_key_exchange->getSharedKey(ecdh_pubkey);
+									std::string shared_key = ecdh_key_exchange->getSharedKey(ecdh_pubkey);
+									if (salt.empty())
+									{
+										errmsg = "Generating ECDH shared key failed";
+									}
+									salt+=shared_key;
 								}
 
 								hmac_key=crypto_fak->generateBinaryPasswordHash(authkey, salt, (std::max)(iterations, client_iterations));
 
 								std::string hmac_loc=crypto_fak->generateBinaryPasswordHash(hmac_key, challenge, 1);								
-								if(hmac_loc==hmac)
+								if(hmac_loc==hmac
+									&& errmsg.empty())
 								{
 									if(id==ID_ISC_AUTH2 || id==ID_ISC_AUTH_TOKEN2)
 									{
@@ -440,7 +446,7 @@ void InternetServiceConnector::ReceivePackets(IRunOtherCallback* run_other)
 									state=ISS_CAPA;	
 									comm_pipe=is_pipe;
 								}
-								else
+								else if(errmsg.empty())
 								{
 									errmsg="Auth failed (Authkey/password wrong)";
 								}
@@ -472,6 +478,10 @@ void InternetServiceConnector::ReceivePackets(IRunOtherCallback* run_other)
 											}
 										}
 									}
+									else
+									{
+										errmsg = "Empty authentication key";
+									}
 								}
 							}
 						}
@@ -483,8 +493,12 @@ void InternetServiceConnector::ReceivePackets(IRunOtherCallback* run_other)
 						CWData data;
 						if(!errmsg.empty())
 						{
-							logFailedAuth(clientname, endpoint_name);
-							Server->Log("Authentication failed in InternetServiceConnector::ReceivePackets: "+errmsg, LL_INFO);
+							if (!token_auth)
+							{
+								logFailedAuth(clientname, endpoint_name);
+							}
+							Server->Log("Authentication failed in InternetServiceConnector::ReceivePackets: "+errmsg 
+								+ (token_auth ? " (token authentication)" :""), LL_INFO);
 							data.addChar(ID_ISC_AUTH_FAILED);
 							data.addString(errmsg);
 						}
@@ -522,7 +536,7 @@ void InternetServiceConnector::ReceivePackets(IRunOtherCallback* run_other)
 #ifndef NO_ZSTD_COMPRESSION
 								if (conn_version == 2)
 								{
-									comp_pipe = new CompressedPipeZstd(comm_pipe, compression_level, -1);
+									comp_pipe = new CompressedPipeZstd(comm_pipe, compression_level);
 								}
 								else
 								{
@@ -898,8 +912,8 @@ std::string InternetServiceConnector::generateOnetimeToken(const std::string &cl
 	std::string ret;
 	ret.resize(sizeof(unsigned int)+token.token.size());
 	token_id=little_endian(token_id);
-	memcpy((char*)ret.data(), &token_id, sizeof(unsigned int));
-	memcpy((char*)ret.data()+sizeof(unsigned int), token.token.data(), token.token.size());
+	memcpy(&ret[0], &token_id, sizeof(unsigned int));
+	memcpy(&ret[sizeof(unsigned int)], token.token.data(), token.token.size());
 	return ret;
 }
 
