@@ -230,6 +230,32 @@ void read_config_file(std::string fn, std::vector<std::string>& real_args)
 				real_args.push_back(strlower(val));
 			}
 		}
+		if (settings->getValue("HTTP_LOCALHOST_ONLY", &val))
+		{
+			val = unquote_value(val);
+
+			if (!val.empty()
+				&& ( val=="1" || 
+					strlower(val)=="true" || 
+					strlower(val)=="yes") )
+			{
+				real_args.push_back("--http_localhost_only");
+				real_args.push_back("1");
+			}
+		}
+		if (settings->getValue("FASTCGI_LOCALHOST_ONLY", &val))
+		{
+			val = unquote_value(val);
+
+			if (!val.empty()
+				&& (val == "1" ||
+					strlower(val) == "true" ||
+					strlower(val) == "yes"))
+			{
+				real_args.push_back("--fastcgi_localhost_only");
+				real_args.push_back("1");
+			}
+		}
 		if (settings->getValue("LOG_ROTATE_FILESIZE", &val))
 		{
 			val = trim(unquote_value(val));
@@ -322,6 +348,94 @@ void read_config_file(std::string fn, std::vector<std::string>& real_args)
 				real_args.push_back(strlower(val));
 			}
 		}
+		if (settings->getValue("INTERNET_MODE_DISABLED", &val))
+		{
+			val = trim(unquote_value(val));
+
+			if (!val.empty() &&
+				 (val == "1" ||
+					strlower(val) == "true" ||
+					strlower(val) == "yes"))
+			{
+				real_args.push_back("--internet_mode_disabled");
+				real_args.push_back("1");
+			}
+		}
+		if (settings->getValue("INTERNET_PORT", &val))
+		{
+			val = trim(unquote_value(val));
+
+			if (!val.empty())
+			{
+				real_args.push_back("--internet_port");
+				real_args.push_back(val);
+			}
+		}
+		if (settings->getValue("INTERNET_LOCALHOST_ONLY", &val))
+		{
+			val = trim(unquote_value(val));
+
+			if (!val.empty() &&
+				(val == "1" ||
+					strlower(val) == "true" ||
+					strlower(val) == "yes"))
+			{
+				real_args.push_back("--internet_localhost_only");
+				real_args.push_back("1");
+			}
+		}
+		if (settings->getValue("INTERNET_DISABLE_WEBSOCKET", &val))
+		{
+			val = trim(unquote_value(val));
+
+			if (!val.empty() &&
+				(val == "1" ||
+					strlower(val) == "true" ||
+					strlower(val) == "yes"))
+			{
+				real_args.push_back("--internet_disable_websocket");
+				real_args.push_back("1");
+			}
+		}
+		if (settings->getValue("FAILED_LOGIN_RATELIMIT", &val))
+		{
+			val = trim(unquote_value(val));
+
+			if (!val.empty() &&
+				(val == "0" ||
+					strlower(val) == "false" ||
+					strlower(val) == "no"))
+			{
+				real_args.push_back("--failed_login_ratelimit");
+				real_args.push_back("0");
+			}
+		}
+		if (settings->getValue("ALLOW_USER_ENUMERATION", &val))
+		{
+			val = trim(unquote_value(val));
+
+			if (!val.empty() &&
+				(val == "0" ||
+					strlower(val) == "false" ||
+					strlower(val) == "no"))
+			{
+				real_args.push_back("--allow_user_enumeration");
+				real_args.push_back("0");
+			}
+		}
+		if (settings->getValue("LOCK_SESSION_TO_IP_AND_USER_AGENT", &val))
+		{
+			val = trim(unquote_value(val));
+
+			if (!val.empty() &&
+				(val == "1" ||
+					strlower(val) == "true" ||
+					strlower(val) == "yes"))
+			{
+				real_args.push_back("--lock_session_to_ip_and_user_agent");
+				real_args.push_back("1");
+			}
+		}
 	}	
 
 	if(destroy_server)
