@@ -102,6 +102,37 @@ namespace
 		std::string session;
 	};
 
+	class RestoreTokenKeepaliveThread : public IThread
+	{
+	public:
+		RestoreTokenKeepaliveThread(std::string token)
+			: do_quit(false), token(token)
+		{
+
+		}
+
+		void operator()()
+		{
+			while (!do_quit)
+			{
+				ServerChannelThread::SRestoreToken token_info;
+				ServerChannelThread::get_restore_token(token, token_info);
+
+				Server->wait(10000);
+			}
+
+			delete this;
+		}
+
+		void doQuit() {
+			do_quit = true;
+		}
+
+	private:
+		volatile bool do_quit;
+		std::string token;
+	};
+
 	class FileservClientThread : public IThread
 	{
 	public:
@@ -128,6 +159,9 @@ namespace
 }
 
 int ServerChannelThread::img_id_offset=0;
+IMutex* ServerChannelThread::restore_token_mutex = NULL;
+std::map<std::string, ServerChannelThread::SRestoreToken> ServerChannelThread::restore_tokens;
+std::map<std::string, ServerChannelThread::SRestoreToken> ServerChannelThread::restore_tokens_used;
 
 void ServerChannelThread::initOffset()
 {
@@ -147,12 +181,41 @@ void ServerChannelThread::initOffset()
     }
 }
 
+void ServerChannelThread::init_mutex()
+{
+	restore_token_mutex = Server->createMutex();
+}
+
 bool ServerChannelThread::isOnline()
 {
 	IScopedLock lock(mutex);
 	return input != NULL;
 }
 
+void ServerChannelThread::add_restore_token(const std::string& token, int backupid)
+{
+	assert(restore_token_mutex != NULL);
+
+	IScopedLock lock(restore_token_mutex);
+
+	SRestoreToken token_info;
+	token_info.backupid = backupid;
+	restore_tokens[token] = token_info;
+
+	timeout_restore_tokens_used();
+}
+
+void ServerChannelThread::remove_restore_token(const std::string& token)
+{
+	IScopedLock lock(restore_token_mutex);
+
+	std::map<std::string, SRestoreToken>::iterator it = restore_tokens.find(token);
+	if (it != restore_tokens.end())
+	{
+		restore_tokens.erase(it);
+	}
+}
+
 ServerChannelThread::ServerChannelThread(ClientMain *client_main, const std::string& clientname, int clientid,
 	bool internet_mode, bool allow_restore,
 	 std::string server_token, const std::string& virtual_client,
@@ -160,7 +223,8 @@ ServerChannelThread::ServerChannelThread(ClientMain *client_main, const std::str
 	client_main(client_main), clientname(clientname), clientid(clientid), settings(NULL),
 		internet_mode(internet_mode), allow_restore(allow_restore), keepalive_thread(NULL), server_token(server_token),
 	virtual_client(virtual_client), allow_shutdown(true),
-	parent(parent), next_reauth_time(0), reauth_tries(0)
+	parent(parent), next_reauth_time(0), reauth_tries(0), restore_token_keepalive_thread(NULL),
+	startup_timestamp(0)
 {
 	do_exit=false;
 	mutex=Server->createMutex();
@@ -197,10 +261,15 @@ void ServerChannelThread::run()
 	{
 		if(input==NULL)
 		{
-			IPipe *np=client_main->getClientCommandConnection(settings, 10000, &client_addr);
+			bool require_reauth = false;
+			IPipe *np=client_main->getClientCommandConnection(settings, 10000, &client_addr, true, &require_reauth);
 			if(np==NULL)
 			{
-				Server->Log("Connecting Channel to "+clientname+" failed - CONNECT error -55", LL_DEBUG);
+				if (require_reauth)
+				{
+					startReauth();
+				}
+				Server->Log("Connecting Channel to "+clientname+" failed - CONNECT error -55 require_reauth="+convert(require_reauth), LL_DEBUG);
 				Server->wait(10000);
 			}
 			else
@@ -212,7 +281,7 @@ void ServerChannelThread::run()
 				curr_ident = client_main->getIdentity();
 				tcpstack.reset();
 				tcpstack.setAddChecksum(client_main->isOnInternetConnection());
-				tcpstack.Send(input, curr_ident +"1CHANNEL capa="+convert(constructCapabilities())+"&token="+server_token+"&restore_version=1&virtual_client="+EscapeParamString(virtual_client));
+				tcpstack.Send(input, curr_ident +"1CHANNEL capa="+convert(constructCapabilities())+"&token="+server_token+"&restore_version=1&startup=1&virtual_client="+EscapeParamString(virtual_client));
 
 				lasttime=Server->getTimeMS();
 				lastpingtime=lasttime;
@@ -288,6 +357,11 @@ void ServerChannelThread::run()
 		keepalive_thread->doQuit();
 	}
 
+	if (restore_token_keepalive_thread != NULL)
+	{
+		restore_token_keepalive_thread->doQuit();
+	}
+
 	if (!fileclient_threads.empty())
 	{
 		Server->Log(clientname+"/server_channel: Waiting for fileclient threads...", LL_DEBUG);
@@ -308,7 +382,8 @@ void ServerChannelThread::doExit(void)
 
 std::string ServerChannelThread::processMsg(const std::string &msg)
 {
-	if (msg != "ERR")
+	if (msg != "ERR" &&
+		msg != "ERR REQUIRE ENC")
 	{
 		reauth_tries = 0;
 		next_reauth_time = 0;
@@ -316,18 +391,7 @@ std::string ServerChannelThread::processMsg(const std::string &msg)
 
 	if(msg=="ERR")
 	{
-		if (next_reauth_time == 0
-			|| Server->getTimeMS() > next_reauth_time)
-		{
-			client_main->forceReauthenticate();
-			client_main->sendToPipe("WAKEUP");
-
-			++reauth_tries;
-
-			unsigned int waittime = (std::min)(static_cast<unsigned int>(1000.*pow(2., static_cast<double>(reauth_tries))), (unsigned int)30 * 60 * 1000);
-			waittime = (std::max)((unsigned int)60000, waittime);
-			next_reauth_time = Server->getTimeMS()+waittime;
-		}
+		startReauth();
 		return std::string();
 	}
 	else if(msg=="START BACKUP INCR")
@@ -427,25 +491,34 @@ std::string ServerChannelThread::processMsg(const std::string &msg)
 		str_map params;
 		ParseParamStrHttp(s_params, &params);
 	}
-	else if(next(msg, 0, "DOWNLOAD IMAGE ") && allow_restore && hasDownloadImageRights())
+	else if(next(msg, 0, "DOWNLOAD IMAGE ") && allow_restore)
 	{
 		std::string s_params=msg.substr(15);
 		str_map params;
 		ParseParamStrHttp(s_params, &params);
 
+		if (hasDownloadImageRights(params))
 		{
-			IScopedLock lock(mutex);
-			allow_shutdown = false;
+			{
+				IScopedLock lock(mutex);
+				allow_shutdown = false;
+			}
+
+			add_extra_channel();
+			DOWNLOAD_IMAGE(params);
+			remove_extra_channel();
+			Server->getDatabase(Server->getThreadID(), URBACKUPDB_SERVER)->destroyAllQueries();
+
+			{
+				IScopedLock lock(mutex);
+				allow_shutdown = true;
+			}
 		}
-
-		add_extra_channel();
-		DOWNLOAD_IMAGE(params);
-		remove_extra_channel();
-		Server->getDatabase(Server->getThreadID(), URBACKUPDB_SERVER)->destroyAllQueries();
-
+		else if (params.find("token") != params.end())
 		{
-			IScopedLock lock(mutex);
-			allow_shutdown = true;
+			Server->Log("Image restore token timeout", LL_DEBUG);
+			_i64 r = -1;
+			input->Write((char*)&r, sizeof(_i64), 30000);
 		}
 	}
 	else if(next(msg, 0, "DOWNLOAD FILES TOKENS "))
@@ -510,6 +583,13 @@ std::string ServerChannelThread::processMsg(const std::string &msg)
 			}
 		}
 	}
+	else if (next(msg, 0, "STARTUP "))
+	{
+		std::string s_params = msg.substr(8);
+		str_map params;
+		ParseParamStrHttp(s_params, &params);
+		STARTUP(params);
+	}
 	else
 	{
 		{
@@ -580,6 +660,14 @@ void ServerChannelThread::LOGIN(str_map& params)
 
 	if(needs_login())
 	{
+		Helper::IpLogin ip_login(client_addr);
+
+		if (helper.rateLimited(client_addr))
+		{
+			tcpstack.Send(input, "err");
+			return;
+		}
+
 		if(session.empty())
 		{
 			session=helper.generateSession("anonymous");
@@ -609,6 +697,7 @@ void ServerChannelThread::LOGIN(str_map& params)
 			helper.getSession()->id=-1;
 			tcpstack.Send(input, "err");
 			logFailedLogin(helper, PARAMS, params["username"], LoginMethod_RestoreCD);
+			helper.addToRateLimit(client_addr);
 		}
 	}
 	else
@@ -666,7 +755,7 @@ void ServerChannelThread::SALT(str_map& params)
 	}
 }
 
-bool ServerChannelThread::hasDownloadImageRights()
+bool ServerChannelThread::hasDownloadImageRights(const str_map& params)
 {
 	if(!needs_login())
 	{
@@ -674,6 +763,12 @@ bool ServerChannelThread::hasDownloadImageRights()
 		return true;
 	}
 
+	if (params.find("token") != params.end())
+	{
+		SRestoreToken token_info;
+		return get_restore_token(params.at("token"), token_info);
+	}
+
 	str_map GET;
 	str_map PARAMS;
 	GET["ses"]=session;
@@ -685,12 +780,6 @@ bool ServerChannelThread::hasDownloadImageRights()
 		return false;
 	}
 
-	if(helper.getSession()==NULL)
-	{
-		Server->Log("Channel session timeout", LL_ERROR);
-		return false;
-	}
-
 	if(helper.getSession()->id==-1)
 	{
 		all_client_rights=false;
@@ -960,12 +1049,45 @@ void ServerChannelThread::DOWNLOAD_IMAGE(str_map& params)
 
 	ServerBackupDao backup_dao(db);
 
-	int img_id=watoi(params["img_id"])-img_id_offset;
-	
-	IQuery *q=db->Prepare("SELECT path, version, clientid, letter FROM backup_images WHERE id=? AND strftime('%s', backuptime)=?");
-	q->Bind(img_id);
-	q->Bind(params["time"]);
-	db_results res=q->Read();
+	bool check_client_permissions = true;
+	db_results res;
+	int img_id;
+	if (params.find("token") != params.end())
+	{
+		SRestoreToken token_info;
+		std::string restore_token = params["token"];
+		if (!get_restore_token(restore_token, token_info))
+		{
+			_i64 r = -1;
+			input->Write((char*)&r, sizeof(_i64), img_send_timeout);
+			return;
+		}
+
+		if (restore_token_keepalive_thread != NULL)
+		{
+			restore_token_keepalive_thread->doQuit();
+		}
+		restore_token_keepalive_thread = new RestoreTokenKeepaliveThread(restore_token);
+		Server->getThreadPool()->execute(restore_token_keepalive_thread, "restore token keepalive");
+
+		check_client_permissions = false;
+
+		IQuery* q = db->Prepare("SELECT path, version, clientid, letter FROM backup_images WHERE id=?");
+		q->Bind(token_info.backupid);
+		res = q->Read();
+
+		img_id = token_info.backupid;
+	}
+	else
+	{
+		img_id = watoi(params["img_id"]) - img_id_offset;
+
+		IQuery* q = db->Prepare("SELECT path, version, clientid, letter FROM backup_images WHERE id=? AND strftime('%s', backuptime)=?");
+		q->Bind(img_id);
+		q->Bind(params["time"]);
+		res = q->Read();
+	}
+
 	if(res.empty())
 	{
 		Server->Log("Image to download not found (img_id=" + convert(img_id) + " time=" + params["time"] + ")", LL_DEBUG);
@@ -978,18 +1100,21 @@ void ServerChannelThread::DOWNLOAD_IMAGE(str_map& params)
 	}
 	else
 	{
-		std::string clientname = get_clientname(db, watoi(res[0]["clientid"]));
-
-		if(clientname.empty()
-			|| !has_restore_permission(clientname, watoi(res[0]["clientid"])))
+		if (check_client_permissions)
 		{
-			Server->Log("No permission to download image of client with id " + res[0]["clientid"], LL_DEBUG);
-			_i64 r=-1;
-			if (!input->Write((char*)&r, sizeof(_i64), img_send_timeout))
+			std::string clientname = get_clientname(db, watoi(res[0]["clientid"]));
+
+			if (clientname.empty()
+				|| !has_restore_permission(clientname, watoi(res[0]["clientid"])))
 			{
-				reset();
+				Server->Log("No permission to download image of client with id " + res[0]["clientid"], LL_DEBUG);
+				_i64 r = -1;
+				if (!input->Write((char*)&r, sizeof(_i64), img_send_timeout))
+				{
+					reset();
+				}
+				return;
 			}
-			return;
 		}
 
 		int img_version=watoi(res[0]["version"]);
@@ -1034,6 +1159,12 @@ void ServerChannelThread::DOWNLOAD_IMAGE(str_map& params)
 				return;
 			}
 		}
+		else
+		{
+			IPipe::SCompressionSettings comp_params;
+			comp_params.mem = IPipe::Compression_HighMem;
+			input->setCompressionSettings(comp_params);
+		}
 
 		uint64 offset=0;
 		str_map::iterator it1=params.find("offset");
@@ -1051,6 +1182,10 @@ void ServerChannelThread::DOWNLOAD_IMAGE(str_map& params)
 		{
 			vhdfile = image_fak->createVHDFile(res[0]["path"], true, 0, 2 * 1024 * 1024, false, IFSImageFactory::ImageFormat_RawCowFile); 
 		}
+		else if (file_extension == "vhdx" || file_extension == "vhdxz")
+		{
+			vhdfile = image_fak->createVHDFile(res[0]["path"], true, 0, 2 * 1024 * 1024, false, IFSImageFactory::ImageFormat_VHDX);
+		}
 		else
 		{
 			vhdfile = image_fak->createVHDFile(res[0]["path"], true, 0);
@@ -1292,7 +1427,7 @@ void ServerChannelThread::DOWNLOAD_FILES( str_map& params )
 		std::vector<std::string> tokens;
 
 		if(create_clientdl_thread(backupid, clientname, clientid, restore_id, status_id, log_id,
-			params["restore_token"], map_paths, clean_other, ignore_other_fs, follow_symlinks, restore_flags, tokens))
+			params["restore_token"], map_paths, clean_other, ignore_other_fs, follow_symlinks, restore_flags, tokens, false))
 		{
 			JSON::Object ret;
 			ret.set("ok", true);
@@ -1432,7 +1567,7 @@ void ServerChannelThread::DOWNLOAD_FILES_TOKENS(str_map& params)
 		if(!create_clientdl_thread(clientname, clientid, clientid, path_info.full_path, path_info.full_metadata_path, filename, 
 			path_info.rel_path.empty(), path_info.rel_path, restore_id, status_id, log_id, params["restore_token"],
 			map_paths, clean_other, ignore_other_fs, greplace(os_file_sep(), "/", path_info.rel_path), follow_symlinks, restore_flags,
-			ticket, tokens, path_info.backup_tokens))
+			ticket, tokens, path_info.backup_tokens, true))
 		{
 			ret.set("err", 5);
 			break;
@@ -1529,6 +1664,21 @@ void ServerChannelThread::RESTORE_DONE( str_map params )
 	
 }
 
+void ServerChannelThread::STARTUP(str_map& params)
+{
+	int64 curr_timestamp = watoi64(params["timestamp"]);
+
+	if (curr_timestamp != startup_timestamp)
+	{
+		if (startup_timestamp != 0)
+		{
+			client_main->updateCapa();
+			client_main->sendToPipe("WAKEUP");
+		}
+		startup_timestamp = curr_timestamp;
+	}
+}
+
 void ServerChannelThread::reset()
 {
 	IScopedLock lock(mutex);
@@ -1603,4 +1753,83 @@ void ServerChannelThread::remove_extra_channel()
 	}
 }
 
+void ServerChannelThread::timeout_restore_tokens_used()
+{
+	int64 ctime = Server->getTimeMS();
+	for (std::map<std::string, SRestoreToken>::iterator it = restore_tokens_used.begin();
+		it != restore_tokens_used.end();)
+	{
+		if (ctime - it->second.lasttime > 10 * 60 * 1000)
+		{
+			std::map<std::string, SRestoreToken>::iterator it_curr = it;
+			++it;
+			restore_tokens_used.erase(it_curr);
+		}
+		else
+		{
+			++it;
+		}
+	}
+}
+
+void ServerChannelThread::startReauth()
+{
+	if (next_reauth_time == 0
+		|| Server->getTimeMS() > next_reauth_time)
+	{
+		client_main->forceReauthenticate();
+		client_main->sendToPipe("WAKEUP");
+
+		++reauth_tries;
+
+		unsigned int waittime = (std::min)(static_cast<unsigned int>(1000. * pow(2., static_cast<double>(reauth_tries))), (unsigned int)30 * 60 * 1000);
+		waittime = (std::max)((unsigned int)60000, waittime);
+		next_reauth_time = Server->getTimeMS() + waittime;
+	}
+}
+
+bool ServerChannelThread::get_restore_token(const std::string& token, SRestoreToken& ret)
+{
+	IScopedLock lock(restore_token_mutex);
+
+	std::map<std::string, SRestoreToken>::iterator it = restore_tokens.find(token);
+	if (it == restore_tokens.end())
+	{
+		it = restore_tokens_used.find(token);
+
+		if (it == restore_tokens_used.end())
+		{
+			return false;
+		}
+		else
+		{
+			const int64 ctime = Server->getTimeMS();
+			if (ctime - it->second.lasttime > 10 * 60 * 1000)
+			{
+				restore_tokens_used.erase(it);
+				return false;
+			}
+
+			it->second.lasttime = ctime;
+		}
+	}
+	else
+	{
+		std::map<std::string, SRestoreToken>::iterator it_used = restore_tokens_used.find(token);
+		if (it_used == restore_tokens_used.end())
+		{
+			it->second.lasttime = Server->getTimeMS();
+			restore_tokens_used[token] = it->second;
+		}
+		else
+		{
+			it_used->second.lasttime = Server->getTimeMS();
+		}
+	}
+
+	ret = it->second;
+
+	return true;
+}
+
 
