@@ -639,7 +639,8 @@ void ServerCleanupThread::do_remove_unknown(void)
 					{
 						std::string extension = findextension(image_files[l].name);
 
-						if (extension != "vhd" && extension != "vhdz" && extension != "raw")
+						if (extension != "vhd" && extension != "vhdz" && extension != "raw"
+							&& extension != "vhdx" && extension != "vhdxz")
 							continue;
 
 						found_image = true;
@@ -694,7 +695,8 @@ void ServerCleanupThread::do_remove_unknown(void)
 			{
 				std::string extension=findextension(cf.name);
 
-				if(extension!="vhd" && extension!="vhdz" && extension!="raw")
+				if(extension!="vhd" && extension!="vhdz" && extension!="raw" &&
+					extension != "vhdx" && extension != "vhdxz")
 					continue;
 
 				bool found=false;
@@ -881,9 +883,11 @@ bool ServerCleanupThread::cleanup_one_imagebackup_client(int clientid, int64 min
 {
 	ServerSettings settings(db, clientid);
 
+	std::string val_info = "max_image_full";
 	int max_image_full=settings.getSettings()->max_image_full;
 	if(minspace!=-1)
 	{
+		val_info = "min_image_full";
 		max_image_full=settings.getSettings()->min_image_full;
 	}
 
@@ -891,7 +895,7 @@ bool ServerCleanupThread::cleanup_one_imagebackup_client(int clientid, int64 min
 
 	int backupid;
 	int full_image_num=(int)getImagesFullNum(clientid, backupid, notit);
-	ServerLogger::Log(logid, "Client with id="+convert(clientid)+" has "+convert(full_image_num)+" full image backups max="+convert(max_image_full), LL_DEBUG);
+	ServerLogger::Log(logid, "Client with id="+convert(clientid)+" has "+convert(full_image_num)+" full image backups "+ val_info +"="+convert(max_image_full), LL_DEBUG);
 	while(full_image_num>max_image_full
 		&& full_image_num>0)
 	{
@@ -940,12 +944,16 @@ bool ServerCleanupThread::cleanup_one_imagebackup_client(int clientid, int64 min
 
 	notit.clear();
 
+	val_info = "max_image_incr";
 	int max_image_incr=settings.getSettings()->max_image_incr;
-	if(minspace!=-1)
-		max_image_incr=settings.getSettings()->min_image_incr;
+	if (minspace != -1)
+	{
+		val_info = "min_image_incr";
+		max_image_incr = settings.getSettings()->min_image_incr;
+	}
 
 	int incr_image_num=(int)getImagesIncrNum(clientid, backupid, notit);
-	ServerLogger::Log(logid, "Client with id="+convert(clientid)+" has "+convert(incr_image_num)+" incremental image backups max="+convert(max_image_incr), LL_DEBUG);
+	ServerLogger::Log(logid, "Client with id="+convert(clientid)+" has "+convert(incr_image_num)+" incremental image backups "+val_info +"="+convert(max_image_incr), LL_DEBUG);
 	while(incr_image_num>max_image_incr
 		&& incr_image_num>0)
 	{
@@ -1285,15 +1293,19 @@ bool ServerCleanupThread::cleanup_one_filebackup_client(int clientid, int64 mins
 
 	int max_file_full=settings.getSettings()->max_file_full;
 	int max_file_incr=settings.getSettings()->max_file_incr;
+	std::string full_val_info = "max_file_full";
+	std::string incr_val_info = "max_file_incr";
 	if(minspace!=-1)
 	{
 		max_file_full=settings.getSettings()->min_file_full;
 		max_file_incr=settings.getSettings()->min_file_incr;
+		full_val_info = "min_file_full";
+		incr_val_info = "min_file_incr";
 	}
 
 	int backupid;
 	int full_file_num=(int)getFilesFullNum(clientid, backupid);
-	ServerLogger::Log(logid, "Client with id="+convert(clientid)+" has "+convert(full_file_num)+" full file backups max="+convert(max_file_full), LL_DEBUG);
+	ServerLogger::Log(logid, "Client with id="+convert(clientid)+" has "+convert(full_file_num)+" full file backups "+ full_val_info +"="+convert(max_file_full), LL_DEBUG);
 	while(full_file_num>max_file_full
 		&& full_file_num>0
 		&& !(full_file_num==1
@@ -1321,7 +1333,7 @@ bool ServerCleanupThread::cleanup_one_filebackup_client(int clientid, int64 mins
 	}
 
 	int incr_file_num=(int)getFilesIncrNum(clientid, backupid);
-	ServerLogger::Log(logid, "Client with id="+convert(clientid)+" has "+convert(incr_file_num)+" incremental file backups max="+convert(max_file_incr), LL_DEBUG);
+	ServerLogger::Log(logid, "Client with id="+convert(clientid)+" has "+convert(incr_file_num)+" incremental file backups "+ incr_val_info +"="+convert(max_file_incr), LL_DEBUG);
 	while(incr_file_num>max_file_incr
 		&& incr_file_num>0)
 	{
@@ -2003,16 +2015,16 @@ void ServerCleanupThread::enforce_quotas(void)
 	{
 		ServerLogger::Log(logid, "Enforcing quota for client \"" + (clients[i].name)+ "\" (id="+convert(clients[i].id)+")", LL_INFO);
 		std::ostringstream log;
-		log << "Quota enforcement report for client \"" << (clients[i].name) << "\" (id=" << clients[i].id  << ")" << std::endl;
+		log << "Quota enforcement report for client \"" << (clients[i].name) << "\" (id=" << clients[i].id  << ")\r\n";
 
 		if(!enforce_quota(clients[i].id, log))
 		{
 			ClientMain::sendMailToAdmins("Quota enforcement failed", log.str());
-			ServerLogger::Log(logid, log.str(), LL_ERROR);
+			ServerLogger::Log(logid, trim(log.str()), LL_ERROR);
 		}
 		else
 		{
-			ServerLogger::Log(logid, log.str(), LL_DEBUG);
+			ServerLogger::Log(logid, trim(log.str()), LL_DEBUG);
 		}
 	}
 
@@ -2030,7 +2042,7 @@ bool ServerCleanupThread::enforce_quota(int clientid, std::ostringstream& log)
 	std::string client_quota = trim(client_settings.getSettings()->client_quota);
 	if(client_quota.empty() || client_quota=="100%" || client_quota=="-")
 	{
-		log << "Client does not have a quota or quota is 100%" << std::endl;
+		log << "Client does not have a quota or quota is 100%\r\n";
 		return true;
 	}
 
@@ -2040,22 +2052,22 @@ bool ServerCleanupThread::enforce_quota(int clientid, std::ostringstream& log)
 		ServerCleanupDao::CondInt64 used_storage=cleanupdao->getUsedStorage(clientid);
 		if(!used_storage.exists || used_storage.value<0)
 		{
-			log << "Error getting used storage of client" << std::endl;
+			log << "Error getting used storage of client\r\n";
 			return false;
 		}
 
 		int64 client_quota=cleanup_amount(client_settings.getSettings()->client_quota, db);
 
-		log << "Client uses " << PrettyPrintBytes(used_storage.value) << " and has a quota of " << PrettyPrintBytes(client_quota) << std::endl;
+		log << "Client uses " << PrettyPrintBytes(used_storage.value) << " and has a quota of " << PrettyPrintBytes(client_quota) << "\r\n";
 
 		if(used_storage.value<=client_quota)
 		{		
-			log << "Client within assigned quota." << std::endl;
+			log << "Client within assigned quota.\r\n";
 			return true;
 		}
 		else
 		{
-			log << "This requires enforcement of the quota." << std::endl;
+			log << "This requires enforcement of the quota.\r\n";
 		}
 
 		did_remove_something=false;
@@ -2068,7 +2080,7 @@ bool ServerCleanupThread::enforce_quota(int clientid, std::ostringstream& log)
 
 			if(available_space==-1)
 			{
-				log << "Error getting free space -5" << std::endl;
+				log << "Error getting free space -5\r\n";
 				return false;
 			}
 
@@ -2078,7 +2090,7 @@ bool ServerCleanupThread::enforce_quota(int clientid, std::ostringstream& log)
 
 			if(target_minspace<0)
 			{
-				log << "Error. Target space is negative" << std::endl;
+				log << "Error. Target space is negative\r\n";
 				return false;
 			}
 
@@ -2090,7 +2102,7 @@ bool ServerCleanupThread::enforce_quota(int clientid, std::ostringstream& log)
 				{
 					if (cleanup_one_imagebackup_client(clientid, target_minspace, imagebid))
 					{
-						log << "Removed image backupd with id " << imagebid << std::endl;
+						log << "Removed image backupd with id " << imagebid << "\r\n";
 						did_remove_something = true;
 						//TODO: wait here for btrfs subvol remove to finish
 						if (hasEnoughFreeSpace(target_minspace, &client_settings))
@@ -2117,7 +2129,7 @@ bool ServerCleanupThread::enforce_quota(int clientid, std::ostringstream& log)
 				int filebid;
 				if(cleanup_one_filebackup_client(clientid, target_minspace, filebid))
 				{
-					log << "Removed file backup with id " << filebid << std::endl;
+					log << "Removed file backup with id " << filebid << "\r\n";
 
 					did_remove_something=true;
 					nopc=0;
@@ -2768,7 +2780,8 @@ bool ServerCleanupThread::cleanup_clientlists()
 		{
 			int backupid = watoi(getbetween("clientlist_b_", ".ub", files[i].name));
 
-			if(cleanupdao->hasMoreRecentFileBackup(backupid).exists)
+			if(!cleanupdao->getFileBackupInfo(backupid).exists ||
+				cleanupdao->hasMoreRecentFileBackup(backupid).exists)
 			{
 				if(!Server->deleteFile(os_file_prefix(srcfolder+os_file_sep()+files[i].name)))
 				{
