@@ -282,17 +282,8 @@ namespace backupaccess
 {
 	std::string getBackupFolder(IDatabase* db)
 	{
-		IQuery* q=db->Prepare("SELECT value FROM settings_db.settings WHERE key='backupfolder' AND clientid=0");
-		db_results res_bf=q->Read();
-		q->Reset();
-		if(!res_bf.empty() )
-		{
-			return res_bf[0]["value"];
-		}
-		else
-		{
-			return std::string();
-		}
+		ServerSettings settings(db);
+		return settings.getSettings()->backupfolder;
 	}
 
 	std::string decryptTokens(IDatabase* db, const str_map& CURRP)
@@ -984,6 +975,10 @@ namespace backupaccess
 							ret.set("volume_name", mbr.volume_name);
 							ret.set("fs_type", mbr.fsn);
 							ret.set("serial_number", mbr.serial_number);
+							if (mbr.volume_name.find("LINUX:") == 0)
+							{
+								ret.set("linux_image_restore", true);
+							}
 						}
 					}
 				}
@@ -999,6 +994,11 @@ namespace backupaccess
 			{
 				vhdfile.reset(image_fak->createVHDFile(path, true, 0));
 			}
+			else if (extension == "vhdx"
+				|| extension == "vhdxz")
+			{
+				vhdfile.reset(image_fak->createVHDFile(path, true, 0, 2 * 1024 * 1024, false, IFSImageFactory::ImageFormat_VHDX));
+			}
 			else if(extension=="raw")
 			{
 				vhdfile.reset(image_fak->createVHDFile(path, true, 0, 2*1024*1024, false, IFSImageFactory::ImageFormat_RawCowFile));
@@ -1732,7 +1732,31 @@ ACTION_IMPL(backups)
 						{
 							bool has_mount_timeout;
 							std::string mount_errmsg;
-							backuppath = ImageMount::get_mount_path(-1*backupid, t_clientid, 0, true, mounted_image, -1, has_mount_timeout, mount_errmsg);
+							int partition = 0;
+							std::string path;
+							std::vector<IFSImageFactory::SPartition> partitions;
+							backupaccess::get_image_info(db, -1*backupid, t_clientid,
+								0, path, partitions);
+
+							if (partitions.size() > 1)
+							{
+								while (!u_path.empty()
+									&& u_path[0] == '/')
+								{
+									u_path.erase(0, 1);
+								}
+								if (next(u_path, 0, "partition "))
+								{
+									std::string part = getbetween("partition ", "/", u_path);
+									if (part.empty())
+									{
+										part = getafter("partition ", u_path);
+									}
+									partition = watoi(part) - 1;
+									u_path = u_path.substr(10 + part.size());
+								}
+							}
+							backuppath = ImageMount::get_mount_path(-1*backupid, t_clientid, partition, true, mounted_image, -1, has_mount_timeout, mount_errmsg);
 							path_info = backupaccess::get_image_path_info(u_path, clientname, backupfolder, backupid, backuppath);
 						}
 
@@ -1790,7 +1814,7 @@ ACTION_IMPL(backups)
 							if(!create_clientdl_thread(clientname, t_clientid, t_clientid, path_info.full_path, path_info.full_metadata_path, CURRP["filter"],
 								path_info.rel_path.empty(), path_info.rel_path, restore_id, status_id, log_id, std::string(),
 								std::vector< std::pair<std::string, std::string> >(), true, true, greplace(os_file_sep(), "/", path_info.rel_path), true,
-								restore_flags, ticket, tokens, path_info.backup_tokens))
+								restore_flags, ticket, tokens, path_info.backup_tokens, false))
 							{
 								ret.set("err", "internal_error");
                                 helper.Write(ret.stringify(false));
